(function(e) {
    var callHybridMonitor = function(data) {
        if ('YixinJSBridge' in window) {
            window.YixinJSBridge.call('hybridMonitor', data);
        }
    };

    e.hybridMonitor = {};

    e.hybridMonitor.getStatistics = function() {
        var navigationValue = JSON.stringify(performanceTiming.getNavigation());
        var pagePerformanceValue = JSON.stringify(performanceTiming.getNavigationTiming());

        if (pagePerformanceValue == '{}') {
            return;
        }

        var data = {
            type: 'statistics',
            statistics: {
                navigation: navigationValue,
                pagePerformance: pagePerformanceValue,
                serviceWorker: 0
            }
        };

        if (e.location.protocol.indexOf('https') != -1) {
            data.statistics.serviceWorker = ('serviceWorker' in e.navigator) ? 1 : -1;
        }

        callHybridMonitor(data);
    };

    var sendResourceTiming = function() {
        if (!('YixinJSBridge' in window)) return;

        var resourceTiming = performanceTiming.getResourceTiming();
        var length = resourceTiming.length;
        if (length > 0) {
            var data = {
                type: 'statistics',
                statistics: {
                    resourcePerformance: JSON.stringify(resourceTiming)
                }
            };

            if (e.performance && e.performance.getEntriesByType) {
                data.statistics.resourceTotalCount = e.performance.getEntriesByType('resource').length;
            }

            callHybridMonitor(data);
        }
    };

    e.setInterval(
        function() {
            sendResourceTiming();
        }, 500
    );

    var performanceTiming = function() {
        function navigation() {
            if (!e.performance || !e.performance.navigation) return {};
            var navigation = e.performance.navigation;
            return {
                redirectCount: navigation.redirectCount,
                type: navigation.type
            }
        };

        function navigationTiming() {
            if (!e.performance || !e.performance.timing) return {};
            var time = e.performance.timing;

            var loadPageValue = time.loadEventEnd - time.navigationStart;
            var redirectValue = time.redirectEnd - time.redirectStart;
            var lookupDomainValue = time.domainLookupEnd - time.domainLookupStart;
            var domReadyValue = time.domComplete - time.responseEnd;
            var ttfbValue = time.responseStart - time.navigationStart;
            var requestValue = time.responseEnd - time.requestStart;
            var loadEventValue = time.loadEventEnd - time.loadEventStart;
            var appCacheValue = time.domainLookupStart - time.fetchStart;
            var connectValue = time.connectEnd - time.connectStart;
            var unloadEventValue = time.unloadEventEnd - time.unloadEventStart;
            var domCompileValue = time.domInteractive - time.domLoading;
            var domContentLoadValue = time.domContentLoadedEventEnd - time.domContentLoadedEventStart;

            if (loadPageValue < 0 ||
                redirectValue < 0 ||
                lookupDomainValue < 0 ||
                domReadyValue < 0 ||
                ttfbValue < 0 ||
                requestValue < 0 ||
                loadEventValue < 0 ||
                appCacheValue < 0 ||
                connectValue < 0 ||
                unloadEventValue < 0 ||
                domCompileValue < 0 ||
                domContentLoadValue < 0) {
                return {};
            }

            return {
                loadPage: loadPageValue,
                redirect: redirectValue,
                lookupDomain: lookupDomainValue,
                domReady: domReadyValue,
                ttfb: ttfbValue,
                request: requestValue,
                loadEvent: loadEventValue,
                appCache: appCacheValue,
                connect: connectValue,
                unloadEvent: unloadEventValue,
                domCompile: domCompileValue,
                domContentLoad: domContentLoadValue
            };
        };

        function resourceTiming(start, end) {
            if (!e.performance || !e.performance.getEntriesByType) return [];
            var time = e.performance.getEntriesByType('resource').slice(start, end);
            var resArr = [];
            for (i = 0; i < time.length; i++) {
                res = {
                    name: time[i].name.length > 1024 ? time[i].name.substr(0, 1023) : time[i].name,
                    initiatorType: time[i].initiatorType,
                    duration: Math.round(time[i].duration),
                };
                resArr.push(res);
            }
            return resArr;
        };

        return {
            sendLimitSize: 7,
            sendResourceTimingIndex: 0,
            getNavigation: function() {
                return navigation();
            },
            getNavigationTiming: function() {
                return navigationTiming();
            },
            getResourceTiming: function() {
                if (!e.performance || !e.performance.getEntriesByType) return [];

                var timing = [];
                var len = e.performance.getEntriesByType('resource').length;

                if (len > (this.sendResourceTimingIndex + this.sendLimitSize)) {
                    var end = this.sendResourceTimingIndex + this.sendLimitSize;
                    timing = resourceTiming(this.sendResourceTimingIndex, end);
                    this.sendResourceTimingIndex = end;
                } else if (len > this.sendResourceTimingIndex) {
                    timing = resourceTiming(this.sendResourceTimingIndex, len);
                    this.sendResourceTimingIndex = len;
                }

                return timing;
            }
        };
    }();

    e.addEventListener('error',
        function(e) {
            var errorMessage = {
                url: e.filename,
                message: e.message,
                lineNo: e.lineno,
                colNo: e.colno
            };

            var data = {
                type: 'error',
                error: JSON.stringify(errorMessage)
            };

            callHybridMonitor(data);
        }, !1);
})(this);